export DYLD_FALLBACK_LIBRARY_PATH='/Applications/ParaView-5.9.1.app/Contents/Libraries'
export LD_LIBRARY_PATH='/Applications/ParaView-5.9.1.app/Contents/Libraries'
export DYLD_FALLBACK_FRAMEWORK_PATH='/Applications/ParaView-5.9.1.app/Contents/Frameworks'
export PYTHONPATH=/Applications/ParaView-5.9.1.app/Contents/Python:/Applications/ParaView-5.9.1.app/Contents/Libraries